/*

The NetUsage Project
http://netusage.mozdev.org/

Copyright (C) 2006, NetUsage Project
This extension can be freely distributed under
the terms of the Mozilla Public Licence 1.1
See the licence at http://www.mozilla.org/MPL/

---------
Net Usage Item
XPCOM

*/

const nui_contract_id = "@mozilla.org/netusageitem;1";
const nui_class_id = "{489e3485-7a4a-4438-9185-bf14c0c317ba}";



nsNetUsageItem.prototype = {

	//
	// nsINetUsageItem
	//

	get itemid() { return this.mitemid; },
	get status() { return this.mStatus; },
	get errorText() { return this.mErrorText; },
	get lastFetched() { return this.mLastFetched; },
	get ispid() { return this.isp ? this.isp.id : ''; },
	getVar: function(k) { return this.vars[k]; },

	//
	// private
	//

	init: function(aitemid) {
		// includes
		var scriptinc = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService().QueryInterface(Components.interfaces.mozIJSSubScriptLoader);
		if (typeof(__ISPS_JS__) == 'undefined')
			scriptinc.loadSubScript("chrome://netusageitem/content/isps.js");
		if (typeof(__USAGELIB_JS__) == 'undefined')
			scriptinc.loadSubScript("chrome://netusageitem/content/usagelib.js");
		this.getISPInterface = nuGetISPInterface;

		this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].getService().QueryInterface(Components.interfaces.nsIObserverService);

		ndump('XXXX init: '+aitemid);

		this.mitemid = aitemid;
		this.checkInterval = -100;
		this.isp = null;
		this.mLastFetched = 0;
		
		this.timer = Components.classes['@mozilla.org/timer;1'].createInstance(Components.interfaces.nsITimer);

		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		//var pref = prefSvc.getBranch("extensions.netusage."+this.mitemid+".");
		var pref = prefSvc.getBranch(""); // TODO why doesn't above work?
		pref.QueryInterface(Components.interfaces.nsIPrefBranch2);
		pref.addObserver("extensions.netusage."+this.mitemid+".", this, false);
		ndump("extensions.netusage."+this.mitemid+".");
		
		
		// load prefs should trigger initTimer()
		this.loadPrefs();
	},
	
	uninit: function() {
		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefSvc.getBranch("");
		pref.QueryInterface(Components.interfaces.nsIPrefBranch2);
		pref.removeObserver("extensions.netusage."+this.mitemid+".", this);
		this.timer.cancel();
		this.abort();
		//this.observerSvc.removeObserver(this, '');
		this.observerSvc = null;
		this.timer = null;
		ndump('UnInit !!!!!!!!!!!!!!!!');
	},

	// initialises timer
	initTimer: function () {
		this.timer.cancel();
		if (this.isp != null && this.checkInterval > 0)
			this.timer.init(this, this.checkInterval * 1000, this.timer.TYPE_REPEATING_SLACK);
		else
			ndump('timer disabled');
	},

	//
	// nsINetUsageItem
	//
	
	setISPInterface: function(objjson) {
		this.isp = eval('(' + objjson + ')');
	},
	
	getVarsObjStr: function(s) {
		return ndumpobjstr(this.vars);
	},
	
	getVarExpand: function(s) {
		return nuVarExpand(s, this.vars, false);
	},
	
	abort: function() {
		if (this.mStatus == 'fetching') {
			// abort
			ndump('aborting...');
			this.xmlhttp.abort();
			this.endUpdate('aborted', 'User aborted usage update');
		}
	},
	
	endUpdate: function(status, errorText) {
		ndump('Fetch ended');
		if (status != null)
			this.mStatus = status;
		if (errorText != null)
			this.mErrorText = errorText;
		try {
		this.observerSvc.removeObserver(this, 'http-on-examine-response');
		} catch(e) {}
		this.observerSvc.notifyObservers(this, 'netusage-update-gui', 'draw');
		this.xmlhttp = null;
		this.mDebughttp = '';
	},

	updateUsage: function() {
		try {

		if (this.isp == null) {
			ndump('updateUsage this.isp == null');
			this.mErrorText = 'Please select an ISP in your preferences.'
			return false;
		}
		
		if (this.mStatus == 'fetching') {
			// we're already fetching!
			return false;
		}
	
		this.mDebughttp = '';
		this.mStatus = 'fetching';
		// beyond this point, we will always return true and we will always notify to show error //
		
		// update all client displays (so they flash)
		this.observerSvc.notifyObservers(this, 'netusage-update-gui', 'draw');

		// clean step, vars and usagedata
		this.step = 0;
		this.nextstep = 0;
		this.vars = new Array();
		this.vars['ispname'] = this.isp.name;
		if (this.isp.vars) {
			// copy over isp vars
			for (i in this.isp.vars)
				this.vars[i] = this.isp.vars[i];
		}

		// fetch username and password
		var ublogin;
		if (!(ublogin = nuLoadPass('net:usage:bar:'+this.mitemid))) {
			this.endUpdate('error-up', "Username and Password required");
			return true;
		}
		this.vars['USERNAME'] = ublogin[1];
		this.vars['PASSWORD'] = ublogin[2];

		// TODO set this here or when we finish?
		this.mLastFetched = Math.floor((new Date()).getTime()/1000);
		
		// load [override] vars from pref
		try {
		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefSvc.getBranch("extensions.netusage."+this.mitemid+".vars.");
		var childs = pref.getChildList("", { });
		for (var i = 0; i < childs.length; ++i) {
			try {
			ndump('gp '+childs[i]);
			switch(pref.getPrefType(childs[i])) {
				case Components.interfaces.nsIPrefBranch.PREF_STRING:
					this.vars[childs[i]] = pref.getCharPref(childs[i]);
					break;
				case Components.interfaces.nsIPrefBranch.PREF_INT:
					this.vars[childs[i]] = pref.getIntPref(childs[i]);
					break;
				case Components.interfaces.nsIPrefBranch.PREF_BOOL:
					this.vars[childs[i]] = pref.getBoolPref(childs[i]);
					break;
			}
			} catch(e) { ndump(e); }
		}
		} catch(e) { ndump(e); }
		
		// debug
		// ndumpobj('this.vars', this.vars);

		if (!('strToInt' in this.isp)) {
			this.isp.strToInt = function(m, s, isp) {
				if (!s) return 0;
				if (typeof(s) == 'number') return s;
				// axe away commas
				s = s.replace(/,/g, '');
				// we're using MB as std unit because getIntPref is 32bit only
				if (isp.intUnit == UNIT_B)
					return Math.round(parseInt(s, 10)/1000000);
				else if (isp.intUnit == UNIT_KB)
					return Math.round(parseInt(s, 10)/1000);
				else if (isp.intUnit == UNIT_GB)
					return Math.round(parseFloat(s)*1000);
				else // assume mbytes
					return parseInt(s, 10);
			}
		}
		if (!('strToDate' in this.isp)) {
			this.isp.strToDate = function(m, s, isp) {
				var month3l = {'jan': 0, 'feb': 1, 'mar': 2, 'apr': 3, 'may': 4, 'jun': 5, 'jul': 6, 'aug': 7, 'sep': 8, 'oct': 9, 'nov': 10, 'dec': 11,
					'january': 0, 'february': 1, 'march': 2, 'april': 3, 'may': 4, 'june': 5, 'july': 6, 'august': 7, 'september': 8, 'october': 9, 'november': 10, 'december': 11};
				
				s = ''+s; // convert s to string (for s.split)
				if (m == 'roll' && s.length <= 2) {
					// roll day - lets make it a full roll date
					var now = new Date();
					var d = parseInt(s, 10);
					var mn = (d < now.getDate()) ? now.getMonth()+1+1 : now.getMonth()+1;
					s = d + '/' + mn + '/' + now.getFullYear();
				}

				// generic 25/10/2004 or 2004/10/25 or 2004-10-25
				var xpdtar, xtmp;
				xpdtar = s.split(/[-\/ ,]+/);
				if (parseInt(xpdtar[0], 10) > 1000) {
					// we have 2004/10/25: swap first and last date element
					xtmp = xpdtar[0];
					xpdtar[0] = xpdtar[2];
					xpdtar[2] = xtmp;
				}
				// check if 3+ letter months ie Jan/Feb/etc (27 May 2006 or 3 February, 2006)
				if (xpdtar[1].length >= 3)
					xpdtar[1] = month3l[xpdtar[1].toLowerCase()];
				else // UGLY F*(&(* JS Date has Months starting with 0
					xpdtar[1] = parseInt(xpdtar[1], 10)-1;
				ndump('strToDate: '+xpdtar[0]+'::'+xpdtar[1]+'+1::'+xpdtar[2]);

				if (m == 'start' || m == 'roll') {
					return new Date(parseInt(xpdtar[2], 10), parseInt(xpdtar[1], 10), parseInt(xpdtar[0], 10), 0, 0, 0);
				}
				else if (m == 'end') {
					return new Date(parseInt(xpdtar[2], 10), parseInt(xpdtar[1], 10), parseInt(xpdtar[0], 10), 23, 59, 59);
				}
			}
		}

		// for nuking WWW-authenticate
		this.observerSvc.addObserver(this, 'http-on-examine-response', false);
		
		// start the process
		this.stepUpdateUsage();
		
		if (this.mStatus != 'fetching') {
			// error occured, most likely preprocess
			this.endUpdate(null, null);
		}
		
		} catch(e) {
			this.endUpdate('error', 'Exception: ' + e);
		}
		
		return true;
	},

	// steps this.isp.process
	stepUpdateUsage: function() {
		var v;

		ndump("stepUpdateUsage() step "+this.step);
		
		// check loop LOOP MAX = 5
		if (typeof(this.vars['step' + this.step + 'count']) == 'undefined') {
			this.vars['step' + this.step + 'count'] = 0;
		}
		this.vars['step' + this.step + 'count']++;
		
		if (this.vars['step' + this.step + 'count'] > 5) {
			ndump("stepUpdateUsage() step loop maximum exceeded");
			this.debugDump("Died at STEP"+this.step+" stepUpdateUsage() step loop maximum exceeded!!!\n");
			
			this.mStatus = 'error-process';
			this.mErrorText = 'Step looping maximum exceeded!';
			return false;
		}

		// preprocess
		if (typeof(this.isp.process[this.step * 3 + 0]) == 'function')
			(this.isp.process[this.step * 3 + 0])(this.vars);

		if (this.vars['errortext']) {
			// preprocess raised an error
			this.debugDump("Died at STEP"+this.step+" Preprocess\n"+this.vars['errortext']+"\n");
			
			this.mStatus = 'error-process';
			this.mErrorText = this.vars['errortext'];
			return false;
		}

		var s = this.isp.process[this.step * 3 + 1];

		// substitute all vars in URL cmd string
		s = nuVarExpand(s, this.vars, true);

		var params = s.split(/ /);
		var poststr;

		if (params[0] == 'POST')
			poststr = params[2];
		else
			poststr = null;

		this.xmlhttp = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].createInstance(Components.interfaces.nsIXMLHttpRequest);

		//this.xmlhttp.QueryInterface(Components.interfaces.nsIDOMEventTarget);
		this.xmlhttp.addEventListener('load', this, false);
		this.xmlhttp.addEventListener('error', this, false);
		
		ndump('OPEN '+params[0]+';'+params[1]);
		this.xmlhttp.open(params[0] == 'GETBASIC' ? 'GET' : params[0], params[1]);
		
		// throw off caching
		this.xmlhttp.setRequestHeader("If-Modified-Since", "Sat, 1 Jan 2000 00:00:00 GMT");

		if (params[0] == 'POST')
			this.xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
		if (params[0] == 'GETBASIC') {
			this.xmlhttp.setRequestHeader('Authorization', 'Basic ' + nuEncodeBase64(this.vars['USERNAME']+':'+this.vars['PASSWORD']));
			params[0] = 'GET';
		}
		this.mDebughttp += this.step+') '+params[0]+' '+params[1]+' '+(poststr != null ? poststr : '')+"\n";
		this.xmlhttp.send(poststr);

	},
	
	debugDump: function(strprefix) {
		// dump response, source into profile
		var doc = strprefix+this.mDebughttp+"\nVersion "+nuGetVersion()+"\n"+ndumpobjstr('this.vars', this.vars)+"\nNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN\n";
		if (this.xmlhttp)
			doc += this.xmlhttp.responseText;
		
		// sanitize transcript to protect u/p
		if (typeof(this.vars['USERNAME']) != 'undefined' && this.vars['USERNAME'] != null && this.vars['USERNAME'] != '') {
			doc = doc.replace(new RegExp(nuEscapeRegexp(encodeURIComponent(this.vars['USERNAME'])), 'g'), 'UXXX');
			doc = doc.replace(new RegExp(nuEscapeRegexp(this.vars['USERNAME']), 'g'), 'UXXX');
		} 
		if (typeof(this.vars['PASSWORD']) != 'undefined' && this.vars['PASSWORD'] != null && this.vars['PASSWORD'] != '') {
			doc = doc.replace(new RegExp(nuEscapeRegexp(encodeURIComponent(this.vars['PASSWORD'])), 'g'), 'PXXX');
			doc = doc.replace(new RegExp(nuEscapeRegexp(this.vars['PASSWORD']), 'g'), 'PXXX');
		}
		
		var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		file.append("netusage_debug.txt");
		var fs = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		fs.init(file, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
		fs.write(doc, doc.length);
		fs.close();
	},

	// parsing, called by dom event load
	parse: function() {
		var rv, i;

		var ae = this.isp.process[this.step * 3 + 2];
		if (ae == null) {
			// no parsing, skip
			rv = [];
		}
		else if (typeof(ae) == 'object') {
			ndump("ae typeof object/array");
			
			// 3rd arg is an array... loop through it
			for(i in ae) {
				// theoretically, we can mix xpath expr and regexp expr leafs
				if (typeof(ae[i]['exec']) == 'function') {
					// one regexp out of multiple (regexp, xpath)
					ndump("one regexp out of multiple");
					rv = this.parseRegexp(ae[i], this.xmlhttp.responseText);
				}
				else if (typeof(ae[i]) == 'object') {
					// one xpath out of multiple (regexp, xpath)
					ndump("one xpath out of multiple");
					rv = this.parseXML(ae[i], this.xmlhttp.responseXML);
				}
				else if (typeof(ae[i]) == 'string') {
					// only 1 xpath branch, (one xpath branch is a string array of xp expr)
					ndump("1 xpath branch");
					rv = this.parseXML(ae, this.xmlhttp.responseXML);
					this.vars[this.step] = 0; // the branch we took
					break;
				}
				// did we find a match?
				if (rv) {
					this.vars[this.step] = i; // the branch we took
					break;
				}
			}
		}
		else if (typeof(ae['exec']) == 'function') {
			// only 1 regexp
			ndump("ae[exec] typeof regexp");
			rv = this.parseRegexp(ae, this.xmlhttp.responseText);
			this.vars[this.step] = 0;
		}

		if (!rv) {
			// parsing failed
			ndump("parsing failed");
			
			this.debugDump("Died at STEP"+this.step+"\n");
			
			this.mStatus = 'error-parse';
			this.mErrorText = 'A parsing error occured';
			return false;
		}
		
		// move stuff in rv into this.vars, append 'step:' to key
		for(i in rv) {
			this.vars[this.step + ":" + i] = rv[i];
		}

		// test if we isp error'd and determine NEXT STEP
		if (this.isp.branching && typeof(this.isp.branching[this.step+','+this.vars[this.step]]) != 'undefined') {
			if (this.isp.branching[this.step+','+this.vars[this.step]] == null) {
				if (this.vars[(this.step) + ":1"]) {
					this.mErrorText = 'ISP Error: '+this.vars[(this.step) + ":1"];
				} else {
					this.mErrorText = 'Login Failed. Please check your username and password.';
				}
				this.mStatus = 'error-isp';
				this.nextstep = -1;
				return false;
			} else {
				this.nextstep = parseInt(this.isp.branching[this.step+','+this.vars[this.step]]);
				if (this.nextstep > Math.floor(this.isp.process.length/3)) {
					// if next step is greater than step count, end condition
					this.nextstep = Math.floor(this.isp.process.length/3)
				}
			}
		} else
			this.nextstep = this.step + 1;
		
		ndump('nextstep: '+this.nextstep);
		return true;
	},


	// parsing text using regexp
	parseRegexp: function(aExp, aDoc) {
		var a, r;
		r = aExp;

		a = r.exec(aDoc);
		if (!a) {
			ndump('parseRegexp failed');
			return null;
		}
		a[0] = 0;
		return a;
	},


	// parsing XML using xpath
	parseXML: function(aPaths, aDoc) {
		if (aPaths['XPATHVERIFY'] && !aDoc.evaluate(aPaths['XPATHVERIFY'], aDoc, null, 9, null).singleNodeValue) {
			ndump('parse XPATH failed');
			return null;
		}
		for (var k in aPaths) {
			if (k == 'XPATHVERIFY') continue;
			// don't set if already exists (for pref override to work)
			if (typeof(this.vars[k]) == 'undefined') {
				this.vars[k] = aDoc.evaluate(aPaths[k], aDoc, null, 2, null).stringValue;
				ndump('Evalutated '+k+' to '+this.vars[k]);
			}
		}
		return [];
	},
	
	// get the start and end dates as an array [start, end]
	getStartEndDate: function(m, v1, v2) {
		var startdate, enddate;
		var now = new Date();

		if (m == 'roll' || m == 'end') {
			if (m == 'roll') {
				// only roll date AKA next start date specified
				rolldate = this.isp.strToDate(m, v1, this.isp);
				enddate = new Date(rolldate.getTime()-1);
			} else {
				// only end date specified
				enddate = this.isp.strToDate('end', v1, this.isp);
			}
			startdate = new Date(enddate.getFullYear(), enddate.getMonth()-1, enddate.getDate()+1, 0, 0, 0);
		}
		else if (m == 'start') {
			// only start date specified
			startdate = this.isp.strToDate('start', v1, this.isp);
			enddate = new Date(startdate.getFullYear(), startdate.getMonth()+1, startdate.getDate()-1, 23, 59, 59);
		}
		else if (v1 && v2) {
			// both start and end date specified
			startdate = this.isp.strToDate('start', v1, this.isp);
			enddate = this.isp.strToDate('end', v2, this.isp);
		}
		else {
			// calendar month
			startdate = new Date(now.getFullYear(), now.getMonth(), 1, 0, 0, 0);
			enddate = new Date(now.getFullYear(), now.getMonth()+1, 0, 23, 59, 59);
		}
		return [startdate, enddate];
	},


	// load user preferences
	loadPrefs: function () {
		var doinittime = false;

		try {
			var prefSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			var pref = prefSvc.getBranch("extensions.netusage."+this.mitemid+".");
			var prefISP = pref.getCharPref("userisp");
			if (prefISP && prefISP != '' && prefISP != 'na') {
				if (this.isp == null || this.isp.id != prefISP)
					this.isp = this.getISPInterface(prefISP);

			} else {
				if (this.isp != null) {
					this.isp = null;
					doinittime = true; // disable the timer
				}
			}
			var prefCheckInterval = pref.getIntPref("checkinterval");
			// only change checkinterval if different and if interval 
			// is greater than 10 mins (minimum req, so ISP servers aren't DOS'd)
			if (this.checkInterval != prefCheckInterval) {
				if (prefCheckInterval >= 600)
					this.checkInterval = prefCheckInterval;
				else
					this.checkInterval = 3600;
				doinittime = true; // refresh timer interval
			}
			if (doinittime)
				this.initTimer();

		} catch(e) { ndump('SOFTFAIL ' + e); }


	},


	//
	// nsIObserver
	//
	observe: function (subject, topic, data) {
		ndump('Observed: '+subject+' :: '+topic+' ::: '+data);
		switch (topic) {
			case 'timer-callback':
				this.updateUsage();
				break;
			case 'nsPref:changed':
				this.loadPrefs();
				break;
			case 'http-on-examine-response':
				subject.QueryInterface(Components.interfaces.nsIHttpChannel);
				// if our channel
				if (this.xmlhttp.channel == subject) {
					// nuke WWW-Authenticate if exist so ugly dialog doesn't pop up if wrong pass
					try {
					if (subject.getResponseHeader('WWW-Authenticate')) {
						subject.setResponseHeader('WWW-Authenticate', '', false);
						ndump('WWW-Authenticate safely contained');
					}
					} catch(e) { ndump('SOFTFAIL: '+e); }
					
					var hcontenttype, hlocation;
					try {
					hcontenttype = subject.getResponseHeader("Content-Type");
					} catch(e) { hcontenttype = '';}
					try {
					hlocation = ' -> '+subject.getResponseHeader("Location");
					} catch(e) { hlocation = ''; }
					
					this.mDebughttp += this.step+') '+hcontenttype+' '+subject.responseStatus+' '+subject.URI.spec+hlocation+"\n";
				}
				break;
		}
	},



	//
	// nsIDOMEventListener
	//
	handleEvent: function(event) {
		ndump("handleEvent: "+event.type);
		
		// check if we aborted
		if (this.mStatus == 'aborted') {
			//exit
			// TODO WE NEVER REACH HERE?
			ndump('aborted');
			//this.observerSvc.notifyObservers(this, 'netusage-update-gui', 'draw');
			return;
		}
		
		try {

		if (event.type == 'load') {
			// XMLHttpLoadEvent
			if (this.xmlhttp.status != 200 && this.xmlhttp.status != 0 /* for file:///*/) {
				// error
				this.mStatus = 'error-isp';
				if (this.xmlhttp.status == 401) {
					// authorization required
					this.mErrorText = 'Incorrect username or password.';
				} else {
					this.mErrorText = 'ISP error occured: '+this.xmlhttp.status+' '+this.xmlhttp.statusText;
				}
			} else {
				// situation normal
				this.parse();
				this.vars['laststep'] = this.step;
				this.step = this.nextstep;
			}
			
			if (this.mStatus != 'fetching') {
				// parse error or wrong u/p occured
				this.endUpdate(null, null);
			}
			else if (this.step == Math.floor(this.isp.process.length/3)) {
				// we've arrived at the finish
				// post process
				if (typeof(this.isp.process[this.step * 3]) == 'function')
					(this.isp.process[this.step * 3])(this.vars);


				// varmapping
				for (i = 0; i < this.step; i++) {
					var a = this.isp.varMap[i+','+this.vars[i]];
					if (!a) continue;
					for(k in a) {
						// don't override if value already present (for pref override to work)
						if (typeof(this.vars[k]) == 'undefined')
							this.vars[k] = this.vars[a[k]];
					}
				}
				
				// TODO improve the system!!!
				// needs to be below varmapping or else there will be nothing to normalize (only regexp numbers)!
				// normalize ints
				var intvars = ['download', 'download-quota', 'downloadpo', 'downloadpo-quota',
					'upload', 'upload-quota', 'uploadpo', 'uploadpo-quota',
					'usage', 'usage-quota', 'usagepo', 'usagepo-quota',
					'free-download', 'free-download-quota', 'free-upload', 'free-upload-quota', 'free-downloadpo', 'free-downloadpo-quota', 'free-uploadpo', 'free-uploadpo-quota',
					'progress1', 'progress1max', 'progress2', 'progress2max'];
				for (i in intvars) {
					if (typeof(this.vars[intvars[i]]) != 'undefined') {
						this.vars[intvars[i]] = this.isp.strToInt(intvars[i], this.vars[intvars[i]], this.isp);
					}
				}

				// start end date inference
				var d;
				if (this.vars['startdate'] && this.vars['enddate']) {
					d = this.getStartEndDate('startend', this.vars['startdate'], this.vars['enddate']);
				} else if (this.vars['rolldate']) {
					d = this.getStartEndDate('roll', this.vars['rolldate'], null);
				} else if (this.vars['enddate']) {
					d = this.getStartEndDate('end', this.vars['enddate'], null);
				} else if (this.vars['startdate']) {
					d = this.getStartEndDate('start', this.vars['startdate'], null);
				} else {
					d = this.getStartEndDate('', null, null);
				}
				this.vars['startdate'] = Math.floor(d[0].getTime()/1000);
				this.vars['enddate'] = Math.floor(d[1].getTime()/1000);
				
				// update last fetched
				this.mLastFetched = Math.floor((new Date()).getTime()/1000);
				
				
				var fieldvars = ['download', 'download-quota', 'downloadpo', 'downloadpo-quota',
					'upload', 'upload-quota', 'uploadpo', 'uploadpo-quota',
					'usage', 'usage-quota', 'usagepo', 'usagepo-quota',
					'free-download', 'free-download-quota', 'free-upload', 'free-upload-quota', 'free-downloadpo', 'free-downloadpo-quota', 'free-uploadpo', 'free-uploadpo-quota'];
				// write into file
				var sout = this.mLastFetched;
				for (i in fieldvars) {
					if (typeof(this.vars[fieldvars[i]]) != 'undefined') {
						sout += ';;'+this.vars[fieldvars[i]];
					} else
						sout += ';;';
				}
				
				// debug
				this.vars['USERNAME'] = 'UXXX';
				this.vars['PASSWORD'] = 'PXXX';
				ndump('this.vars = {');
				for (i in this.vars) {
					var zzz = this.vars[i];
					if (typeof(zzz) == 'string') {
						zzz = zzz.substr(0, 100);
					}
					ndump(i+': '+zzz);
				}
				ndump('}');
				
				// successful, delete debug file // TODO
				var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
				file.append("netusage_debug.txt");
				if (file.exists())
					file.remove(false);

				this.endUpdate('done', '');
			} else {
				this.stepUpdateUsage();
				
				if (this.mStatus != 'fetching') {
					// error occured, most likely preprocess
					this.endUpdate(null, null);
				}
			}
			
		}
		else if (event.type == 'error') {
			// XMLHttpErrorEvent
			this.endUpdate('error', 'Connection error occured');
		}
		
		} catch(e) {
			this.endUpdate('error', 'Exception: '+e);
		}

	},


	


	//
	// nsISupports
	//


	QueryInterface: function (iid) {
		if (!iid.equals(Components.interfaces.nsINetUsageItem) &&
			!iid.equals(Components.interfaces.nsISupports) &&
			!iid.equals(Components.interfaces.nsIObserver) &&
			!iid.equals(Components.interfaces.nsIDOMEventListener)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
}



function nsNetUsageItem() {

	this.xmlhttp = null;
	this.timer = null;

	this.step = 0;
	this.vars = null;
	this.mLastFetched = 0;
	this.mStatus = "";



}





// MODULE
var module = {
	firstTime: true,

	cid: Components.ID(nui_class_id),

	contractId: nui_contract_id,

	registerSelf: function (compMgr, fileSpec, location, type) {
		if (this.firstTime) {
			// defer first time registering, I have no idea why though :/
			this.firstTime = false;
			throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
		}
		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.cid, "Net Usage Component", this.contractId, fileSpec, location, type);
	},

	unregisterSelf: function(compMgr, fileSpec, location) {
		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		compMgr.unregisterFactoryLocation(this.cid, fileSpec);
	},

	canUnload: function(compMgr) {
		return true;
	},

	// getClassObject: returns component's factory object
	getClassObject: function (compMgr, cid, iid) {
		if (!cid.equals(this.cid)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}

		if (!iid.equals(Components.interfaces.nsIFactory)) {
			throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
		}

		return this.factory;
	},

	// factory object
	factory: {
		createInstance: function (outer, iid) {
			if (outer != null)
				throw Components.results.NS_ERROR_NO_AGGREGATION;
			dump('>>>> NUI CREATED\n');
			return (new nsNetUsageItem()).QueryInterface(iid);
		}
	}
};

function NSGetModule(compMgr, fileSpec) {
	return module;
}